@@ -2,7 +2,7 @@
 # -- fundb --
 #-----------------------------
 
-from . import lib, database, cursor, query
+from . import adapters, jsonquery, lib, database, cursor
 
 # alias to Database
 fundb = database.Database
